#ifndef __ASM_MACH_LOONGSON_IRQ_H_
#define __ASM_MACH_LOONGSON_IRQ_H_

#include <boot_param.h>

#define LS_IRC_BASE	LS_CFG_OFF(0x1400)
#define LS_IRC_OFF(x)	(LS_IRC_BASE + x)

#define LS_IRC_ENT(x)	LS_IRC_OFF(x)
#define LS_IRC_ENT_LPC	LS_IRC_ENT(0xa)
#define LS_IRC_ENT_HT1(n)	LS_IRC_ENT(n + 0x18)
#define LS_IRC_ENT_PCI(n)	LS_IRC_ENT(n + 0x4)

#define LS_IRC_EN	LS_IRC_OFF(0x24)
#define LS_IRC_ENSET	LS_IRC_OFF(0x28)
#define LS_IRC_ENCLR	LS_IRC_OFF(0x2c)
#define LS_IRC_BCE	LS_IRC_OFF(0x38)
#define LS_IRC_AUT	LS_IRC_OFF(0x3c)
#define LS_IRC_ISR(n)	LS_IRC_OFF(0x40 + 8 * (n))

#ifdef CONFIG_CPU_LOONGSON3

#define NR_IRQS	(320+32)
/* cpu core interrupt numbers */
#define MIPS_CPU_IRQ_BASE 56

#define LOONGSON_UART_IRQ   (MIPS_CPU_IRQ_BASE + 2) /* uart */
#define LOONGSON_BRIDGE_IRQ (MIPS_CPU_IRQ_BASE + 3) /* CASCADE */
#define LOONGSON_TIMER_IRQ  (MIPS_CPU_IRQ_BASE + 7) /* cpu timer */

#define LOONGSON_HT1_CFG_BASE		ht_control_base
#define LOONGSON_HT1_INT_VECTOR_BASE	LOONGSON_HT1_CFG_BASE + 0x80
#define LOONGSON_HT1_INT_EN_BASE	LOONGSON_HT1_CFG_BASE + 0xa0
#define LOONGSON_HT1_INT_VECTOR(n)	LOONGSON3_REG32(LOONGSON_HT1_INT_VECTOR_BASE, 4 * (n))
#define LOONGSON_HT1_INTN_EN(n)		LOONGSON3_REG32(LOONGSON_HT1_INT_EN_BASE, 4 * (n))
#define LOONGSON_HT1_INT_VECTOR64(n) LOONGSON3_REG64(LOONGSON_HT1_INT_VECTOR_BASE, 8 * (n))
#define LOONGSON_HT1_INTN_EN64(n)		LOONGSON3_REG64(LOONGSON_HT1_INT_EN_BASE, 8 * (n))

#define LOONGSON_INT_ROUTER_OFFSET	0x1400
#define LOONGSON_INT_ROUTER_INTEN	LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x24)
#define LOONGSON_INT_ROUTER_INTENSET	LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x28)
#define LOONGSON_INT_ROUTER_INTENCLR	LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x2c)
#define LOONGSON_INT_ROUTER_ENTRY(n)	LOONGSON3_REG8(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + (n))
#define LOONGSON_INT_ROUTER_LPC		LOONGSON_INT_ROUTER_ENTRY(0x0a)
#define LOONGSON_INT_ROUTER_HT1(n)	LOONGSON_INT_ROUTER_ENTRY((n) + 0x18)
#define LOONGSON_INT_ROUTER_PCI(n)	LOONGSON_INT_ROUTER_ENTRY((n) + 0x4)
#define LOONGSON_INT_ROUTER_BOUNCE	LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x38)
#define LOONGSON_INT_ROUTER_AUTO	LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x3c)
#define LOONGSON_INT_ROUTER_ISR(n)	LOONGSON3_REG32(LOONGSON3_REG_BASE, LOONGSON_INT_ROUTER_OFFSET + 0x40+(n)*8)

#define LOONGSON_INT_COREx_INTy(x, y)	(1<<(x) | 1<<(y+4))	/* route to int y of core x */

#define LS2H_PCH_IRQ_BASE		64

#define SYS_IRQ_BASE (NR_IRQS-32)
#define PCI_IRQ_BASE (SYS_IRQ_BASE+4)

/* CPU ext ioi csr offset */
#define LOONGSON_EXT_IOI_EN64_OFFSET		0x00001600
#define LOONGSON_EXT_IOI_BOUNCE64_OFFSET	0x00001680
#define LOONGSON_EXT_IOI_COREISR64_OFFSET	0x00001800
#define LOONGSON_EXT_IOI_NODEMAP_OFFSET		0x000014A0
#define LOONGSON_EXT_IOI_MAP_OFFSET		0x000014C0
#define LOONGSON_EXT_IOI_ROUTE_OFFSET		0x00001C00

#define LS_IOI_IRQ_VECTORS	256
#define MAX_32ARRAY_SIZE	(LS_IOI_IRQ_VECTORS  >> 5)
#define LS_IOI_INV_CPU_ID_32	0xFFFFFFFF
#define LS_IOI_INV_CPU_ID	0xFF
#define LS_ANYSEND_OTHER_FUNC_OFFSET	(LOONGSON_OTHER_FUNC_OFFSET + 4)
#define LS_ANYSEND_OTHER_FUNC_EXT_IOI	(LOONGSON_OTHER_FUNC_EXT_IOI >> 32)
#define LS_ANYSEND_BLOCK_SHIFT	31
#define LS_ANYSEND_NODE_SHIFT	18
#define LS_ANYSEND_MASK_SHIFT	27
#define LS_ANYSEND_CPU_MASK	0x3FF
#define LS_ANYSEND_DATA_SHIFT	32
#define LS_ANYSEND_IOI_NODEMAP_ITEMS	8
#define LS_ANYSEND_IOI_EN_ITEMS	8
#define LS_ANYSEND_IOI_IPMAP_ITEMS	2
#define LS_ANYSEND_IOI_ROUTE_ITEMS	64
#define LS_ANYSEND_IOI_BOUNCE_ITEMS	8
#define LS_IOI_NODEMAP_BITS_PER_ENTRY	16
#define LS_ANYSEND_IOI_EN32_DATA	0xFFFFFFFF
#define LS_ANYSEND_IOI_IPMAP_DATA	0x02020202 /* route to IP3 */
#define LS_ANYSEND_IOI_ROUTE_DATA	0x11111111 /* route to node 0 core 0 */
#define LS_ANYSEND_IOI_BOUNCE_DATA	0xFFFFFFFF
#define LS_IOI_CPUNODE_SHIFT_IN_ROUTE	4
#define LS_ANYSEND_ROUTE_DATA_POS(n)	(n << 3)
#define LS_ANYSEND_IOI_UPDATE_IRQ_NUM	32

extern void any_send(unsigned int off, unsigned int data, unsigned int data_mask, unsigned int node);
#define EXT_IOI_REGS_INIT(cnt, off, data, node) ({	\
	int x;						\
	for (x = 0; x < cnt; x++)			\
		any_send(off + (x << 2), data, 0, node);	\
})

/* group 0 */
#define LS2H_PCH_ACPI_IRQ		(0 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HPET_IRQ		(1 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART0_IRQ		(2 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART1_IRQ		(3 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART2_IRQ		(4 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_UART3_IRQ		(5 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SPI_IRQ		(6 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_I2C0_IRQ		(7 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_I2C1_IRQ		(8 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_AC97_IRQ		(9 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DMA0_IRQ		(10 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DMA1_IRQ		(11 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DMA2_IRQ		(12 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_LPC_IRQ		(13 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_INT0_IRQ		(14 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_INT1_IRQ		(15 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_INT2_IRQ		(16 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_INT0_IRQ		(17 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_INT1_IRQ		(18 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_INT2_IRQ		(19 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_RTC_TICK_IRQ		(20 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_TOY_TICK_IRQ		(21 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_NAND_IRQ		(22 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SYS_INTN_IRQ		(23 + LS2H_PCH_IRQ_BASE)

/* group 1 */
#define LS2H_PCH_EHCI_IRQ		(32 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_OHCI_IRQ		(33 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_OTG_IRQ		(34 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GMAC0_IRQ		(35 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GMAC1_IRQ		(36 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SATA_IRQ		(37 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPU_IRQ		(38 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_DC_IRQ			(39 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM0_IRQ		(40 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM1_IRQ		(41 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM2_IRQ		(42 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PWM3_IRQ		(43 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT0_IRQ		(44 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT1_IRQ		(45 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT2_IRQ		(46 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT3_IRQ		(47 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT4_IRQ		(48 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT5_IRQ		(49 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT6_IRQ		(50 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HT7_IRQ		(51 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT0_IRQ		(52 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT1_IRQ		(53 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT2_IRQ		(54 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_PCIE_PORT3_IRQ		(55 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_SATA_PHY_IRQ		(56 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_HDA_IRQ		(57 + LS2H_PCH_IRQ_BASE)

#define LS2H_PCIE_PORT0_INTA_BIT	(1 << 20)
#define LS2H_PCIE_PORT1_INTA_BIT	(1 << 21)
#define LS2H_PCIE_PORT2_INTA_BIT	(1 << 22)
#define LS2H_PCIE_PORT3_INTA_BIT	(1 << 23)

/* group 2 */
#define LS2H_PCH_GPIO0_IRQ		(64 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO1_IRQ		(65 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO2_IRQ		(66 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO3_IRQ		(67 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO4_IRQ		(68 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO5_IRQ		(69 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO6_IRQ		(70 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO7_IRQ		(71 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO8_IRQ		(72 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO9_IRQ		(73 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO10_IRQ		(74 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO11_IRQ		(75 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO12_IRQ		(76 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO13_IRQ		(77 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO14_IRQ		(78 + LS2H_PCH_IRQ_BASE)
#define LS2H_PCH_GPIO15_IRQ		(79 + LS2H_PCH_IRQ_BASE)

#define LS2H_PCH_GPIO_IRQ		LS2H_PCH_GPIO0_IRQ
#define LS2H_PCH_GPIO_FIRST_IRQ		LS2H_PCH_GPIO0_IRQ
#define LS2H_PCH_GPIO_IRQ_COUNT		96
#define LS2H_PCH_GPIO_LAST_IRQ		(LS2H_PCH_GPIO_FIRST_IRQ + LS2H_PCH_GPIO_IRQ_COUNT - 1)

#define LS2H_PCH_LAST_IRQ		(160 + LS2H_PCH_IRQ_BASE)

#define LS2H_PCH_KBD_IRQ		1
#define LS2H_PCH_AUX_IRQ		12

struct ls2h_intctl_regs {
	volatile u32 int_isr;
	volatile u32 int_en;
	volatile u32 int_set;
	volatile u32 int_clr;
	volatile u32 int_pol;
	volatile u32 int_edge;
};

/* LS7A IOAPIC IRQ DEFINITIONS */

/* Registers */
#ifdef CONFIG_KVM_GUEST_LS3A3000
#define LS7A_IOAPIC_BASE                    0x900000e010000000
#else
#define LS7A_IOAPIC_BASE                    0x90000e0010000000
#endif

#define LS7A_IOAPIC_ROUTE_ENTRY_OFFSET      0x100
#define LS7A_IOAPIC_INT_MASK_OFFSET         0x20
#define LS7A_IOAPIC_INT_EDGE_OFFSET         0x60
#define LS7A_IOAPIC_INT_CLEAR_OFFSET        0x80
#define LS7A_IOAPIC_INT_STATUS_OFFSET       0x3a0
#define LS7A_IOAPIC_INT_POL_OFFSET          0x3e0
#define LS7A_IOAPIC_HTMSI_EN_OFFSET         0x40
#define LS7A_IOAPIC_HTMSI_VEC_OFFSET        0x200

#define LS7A_IOAPIC_ROUTE_ENTRY             (LS7A_IOAPIC_BASE + LS7A_IOAPIC_ROUTE_ENTRY_OFFSET)
#define LS7A_IOAPIC_INT_STATUS              (LS7A_IOAPIC_BASE + LS7A_IOAPIC_INT_STATUS_OFFSET)
#define LS7A_IOAPIC_INT_MASK                (LS7A_IOAPIC_BASE + LS7A_IOAPIC_INT_MASK_OFFSET)
#define LS7A_IOAPIC_INT_EDGE                (LS7A_IOAPIC_BASE + LS7A_IOAPIC_INT_EDGE_OFFSET)
#define LS7A_IOAPIC_INT_CLEAR               (LS7A_IOAPIC_BASE + LS7A_IOAPIC_INT_CLEAR_OFFSET)
#define LS7A_IOAPIC_INT_POL                 (LS7A_IOAPIC_BASE + LS7A_IOAPIC_INT_POL_OFFSET)
#define LS7A_IOAPIC_HTMSI_EN                (LS7A_IOAPIC_BASE + LS7A_IOAPIC_HTMSI_EN_OFFSET)
#define LS7A_IOAPIC_HTMSI_VEC               (LS7A_IOAPIC_BASE + LS7A_IOAPIC_HTMSI_VEC_OFFSET)

/* IRQs */
#define LS7A_IOAPIC_IRQ_BASE                64
//#define LS7A_IOAPIC_IRQ_BASE                128

#define LS7A_IOAPIC_UART0_OFFSET            8
#define LS7A_IOAPIC_I2C0_OFFSET             9

#define LS7A_IOAPIC_GMAC0_OFFSET            12
#define LS7A_IOAPIC_GMAC0_PMT_OFFSET        13

#define LS7A_IOAPIC_GMAC1_OFFSET            14
#define LS7A_IOAPIC_GMAC1_PMT_OFFSET        15

#define LS7A_IOAPIC_SATA0_OFFSET            16
#define LS7A_IOAPIC_SATA1_OFFSET            17
#define LS7A_IOAPIC_SATA2_OFFSET            18
#define LS7A_IOAPIC_LPC_OFFSET              19

#define LS7A_IOAPIC_PWM0_OFFSET             24
#define LS7A_IOAPIC_PWM1_OFFSET             25
#define LS7A_IOAPIC_PWM2_OFFSET             26
#define LS7A_IOAPIC_PWM3_OFFSET             27
#define LS7A_IOAPIC_DC_OFFSET               28
#define LS7A_IOAPIC_GPU_OFFSET              29

#define LS7A_IOAPIC_PCIE_F0_PORT0_OFFSET    32
#define LS7A_IOAPIC_PCIE_F0_PORT1_OFFSET    33
#define LS7A_IOAPIC_PCIE_F0_PORT2_OFFSET    34
#define LS7A_IOAPIC_PCIE_F0_PORT3_OFFSET    35

#define LS7A_IOAPIC_PCIE_F1_PORT0_OFFSET    36
#define LS7A_IOAPIC_PCIE_F1_PORT1_OFFSET    37

#define LS7A_IOAPIC_PCIE_H_LO_OFFSET        38
#define LS7A_IOAPIC_PCIE_H_HI_OFFSET        39

#define LS7A_IOAPIC_PCIE_G0_LO_OFFSET       40
#define LS7A_IOAPIC_PCIE_G0_HI_OFFSET       41

#define LS7A_IOAPIC_PCIE_G1_LO_OFFSET       42
#define LS7A_IOAPIC_PCIE_G1_HI_OFFSET       43

#define LS7A_IOAPIC_TOY0_OFFSET             44
#define LS7A_IOAPIC_TOY1_OFFSET             45
#define LS7A_IOAPIC_TOY2_OFFSET             46

#define LS7A_IOAPIC_ACPI_INT_OFFSET         47
#define LS7A_IOAPIC_EHCI0_OFFSET            48
#define LS7A_IOAPIC_OHCI0_OFFSET            49
#define LS7A_IOAPIC_EHCI1_OFFSET            50
#define LS7A_IOAPIC_OHCI1_OFFSET            51

#define LS7A_IOAPIC_RTC_INT0_OFFSET         52
#define LS7A_IOAPIC_RTC_INT1_OFFSET         53
#define LS7A_IOAPIC_RTC_INT2_OFFSET         54

#define LS7A_IOAPIC_HPET_INT_OFFSET         55

#define LS7A_IOAPIC_AC97_DMA0_OFFSET        56
#define LS7A_IOAPIC_AC97_DMA1_OFFSET        57
#define LS7A_IOAPIC_AC97_DMA1_OFFSET        57
#define LS7A_IOAPIC_AC97_HDA_OFFSET         58

#define LS7A_IOAPIC_GPIO_HI_OFFSET          59
#define LS7A_IOAPIC_GPIO_0_OFFSET           60
#define LS7A_IOAPIC_GPIO_1_OFFSET           61
#define LS7A_IOAPIC_GPIO_2_OFFSET           62
#define LS7A_IOAPIC_GPIO_3_OFFSET           63

#define LS7A_IOAPIC_UART0_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_UART0_OFFSET        )
#define LS7A_IOAPIC_I2C0_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_I2C0_OFFSET         )
#define LS7A_IOAPIC_GMAC0_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GMAC0_OFFSET        )
#define LS7A_IOAPIC_GMAC0_PMT_IRQ           (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GMAC0_PMT_OFFSET    )
#define LS7A_IOAPIC_GMAC1_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GMAC1_OFFSET        )
#define LS7A_IOAPIC_GMAC1_PMT_IRQ           (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GMAC1_PMT_OFFSET    )
#define LS7A_IOAPIC_SATA0_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_SATA0_OFFSET        )
#define LS7A_IOAPIC_SATA1_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_SATA1_OFFSET        )
#define LS7A_IOAPIC_SATA2_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_SATA2_OFFSET        )
#define LS7A_IOAPIC_LPC_IRQ                 (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_LPC_OFFSET          )
#define LS7A_IOAPIC_PWM0_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PWM0_OFFSET         )
#define LS7A_IOAPIC_PWM1_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PWM1_OFFSET         )
#define LS7A_IOAPIC_PWM2_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PWM2_OFFSET         )
#define LS7A_IOAPIC_PWM3_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PWM3_OFFSET         )
#define LS7A_IOAPIC_DC_IRQ                  (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_DC_OFFSET           )
#define LS7A_IOAPIC_GPU_IRQ                 (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GPU_OFFSET          )

#define LS7A_IOAPIC_PCIE_F0_PORT0_IRQ       (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_F0_PORT0_OFFSET)
#define LS7A_IOAPIC_PCIE_F0_PORT1_IRQ       (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_F0_PORT1_OFFSET)
#define LS7A_IOAPIC_PCIE_F0_PORT2_IRQ       (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_F0_PORT2_OFFSET)
#define LS7A_IOAPIC_PCIE_F0_PORT3_IRQ       (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_F0_PORT3_OFFSET)
#define LS7A_IOAPIC_PCIE_F1_PORT0_IRQ       (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_F1_PORT0_OFFSET)
#define LS7A_IOAPIC_PCIE_F1_PORT1_IRQ       (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_F1_PORT1_OFFSET)
#define LS7A_IOAPIC_PCIE_H_LO_IRQ           (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_H_LO_OFFSET    )
#define LS7A_IOAPIC_PCIE_H_HI_IRQ           (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_H_HI_OFFSET    )
#define LS7A_IOAPIC_PCIE_G0_LO_IRQ          (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_G0_LO_OFFSET   )
#define LS7A_IOAPIC_PCIE_G0_HI_IRQ          (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_G0_HI_OFFSET   )
#define LS7A_IOAPIC_PCIE_G1_LO_IRQ          (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_G1_LO_OFFSET   )
#define LS7A_IOAPIC_PCIE_G1_HI_IRQ          (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_PCIE_G1_HI_OFFSET   )

#define LS7A_IOAPIC_TOY0_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_TOY0_OFFSET         )
#define LS7A_IOAPIC_TOY1_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_TOY1_OFFSET         )
#define LS7A_IOAPIC_TOY2_IRQ                (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_TOY2_OFFSET         )

#define LS7A_IOAPIC_ACPI_INT_IRQ            (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_ACPI_INT_OFFSET     )
#define LS7A_IOAPIC_EHCI0_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_EHCI0_OFFSET        )
#define LS7A_IOAPIC_OHCI0_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_OHCI0_OFFSET        )
#define LS7A_IOAPIC_EHCI1_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_EHCI1_OFFSET        )
#define LS7A_IOAPIC_OHCI1_IRQ               (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_OHCI1_OFFSET        )

#define LS7A_IOAPIC_RTC_INT0_IRQ            (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_RTC_INT0_OFFSET     )
#define LS7A_IOAPIC_RTC_INT1_IRQ            (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_RTC_INT1_OFFSET     )
#define LS7A_IOAPIC_RTC_INT2_IRQ            (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_RTC_INT2_OFFSET     )

#define LS7A_IOAPIC_HPET_INT_IRQ            (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_HPET_INT_OFFSET     )

#define LS7A_IOAPIC_AC97_DMA0_IRQ           (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_AC97_DMA0_OFFSET    )
#define LS7A_IOAPIC_AC97_DMA1_IRQ           (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_AC97_DMA1_OFFSET    )
#define LS7A_IOAPIC_AC97_DMA1_IRQ           (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_AC97_DMA1_OFFSET    )
#define LS7A_IOAPIC_AC97_HDA_IRQ            (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_AC97_HDA_OFFSET     )

#define LS7A_IOAPIC_GPIO_HI_IRQ             (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GPIO_HI_OFFSET      )
#define LS7A_IOAPIC_GPIO_0_IRQ              (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GPIO_0_OFFSET       )
#define LS7A_IOAPIC_GPIO_1_IRQ              (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GPIO_1_OFFSET       )
#define LS7A_IOAPIC_GPIO_2_IRQ              (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GPIO_2_OFFSET       )
#define LS7A_IOAPIC_GPIO_3_IRQ              (LS7A_IOAPIC_IRQ_BASE + LS7A_IOAPIC_GPIO_3_OFFSET       )
#endif
extern unsigned int ls3a_num_msi_irqs;
#define IPI_IRQ_OFFSET 6
#define IRQ_LS3A_MSI_0 0
#define LS3A_NUM_MSI_IRQS	64
#define LS3A_NUM_MSI_IRQ ls3a_num_msi_irqs
#define LS3A_MAX_IO_IRQ		256
#define LS3A_MAX_MSI_IRQ	192
#define LS3A_IOIRQ_SUB_VECTOR	64
#define LS3A_MSI_SUB_VECTOR 128

/* convert between io irq and HT or EXT vector 
 *
 * irq - vector = LS3A_IOIRQ_SUB_VECTOR
 *
 * vector	-------------------------
 * 		|000-063|064-127|128-255|
 * 		-------------------------
 * irq		|064-127|128-191|192-319|
 * 		-------------------------
 * 		|IOPIC	|MSI		|
 * 		-------------------------		
 * */
#define LS3A_IOIRQ2VECTOR(irq)	(irq - LS3A_IOIRQ_SUB_VECTOR)
#define LS3A_VECTOR2IOIRQ(vector)  (vector + LS3A_IOIRQ_SUB_VECTOR) 

/*
 * Define irq_chip model
 * 
 * */
enum irq_chip_model {
	ICM_CORE = 0, 	/* for single core, such as MIPS */
	ICM_CPU,	/* for cpu cores shared, such as Loongson */
	ICM_PCI_MSI,	/* for pci msi managed through cpu HT vector */
	ICM_PCI_MSI_EXT,	/* for pci msi managed through cpu EXT vector */
	ICM_IOPIC_LINE,		/* for iopic managed through iopic interrupt related registers */
	ICM_IOPIC_HT,		/* for iopic managed through cpu HT vector */
	ICM_IOPIC_HT_EXT,	/* for iopic managed through cpu EXT vector */
	ICM_END
};

#define RS780_PCH_ACPI_IRQ  7
#include_next <irq.h>
#endif /* __ASM_MACH_LOONGSON_IRQ_H_ */
